declare const corrupt: (unreachable: never) => never;

type AnyFunction = (...args: any[]) => unknown;
type ExhaustiveFallback = {
    /**
     * Default fallback
     *
     * @description
     * When declared, "exhaustive" will fallback to this callback
     * instead of throwing an unreachable error
     */
    _?: () => any;
};
/**
 * Ensures no extra values are passed to the object
 */
type ValidateKeys<T, U> = [keyof T] extends [keyof U] ? T : {
    [Key in keyof U]: Key extends keyof T ? T[Key] : never;
};
interface Exhaustive {
    <Union extends string, Match extends ExhaustiveUnion<Union> = ExhaustiveUnion<Union>>(union: Union, match: ValidateKeys<Match, ExhaustiveUnion<Union>>): Match[keyof Match] extends AnyFunction ? ReturnType<Match[keyof Match]> : never;
    _tag: <Union extends object, Tag extends keyof Union, Match extends ExhaustiveTag<Union, Tag> = ExhaustiveTag<Union, Tag>>(union: Union, tag: Tag, match: ValidateKeys<Match, ExhaustiveTag<Union, Tag>>) => Match[keyof Match] extends AnyFunction ? ReturnType<Match[keyof Match]> : never;
}
type ExhaustiveUnion<Union extends string> = {
    [Key in Union]: (value: Key) => any;
} & ExhaustiveFallback;
type ExhaustiveTag<Union extends object, Tag extends keyof Union> = {
    [Key in Union[Tag] & string]: (value: Extract<Union, {
        [K in Tag]: Key;
    }>) => any;
} & ExhaustiveFallback;
declare const exhaustive: Exhaustive;

export { corrupt, exhaustive };
